#include "engine.h"
#include <time.h>

SDL_Surface* background;
SDL_Surface* pipe;
SDL_Surface* ground;
SDL_Surface* bird;
Effect* flapSound;
Effect* hitSound;
Effect* pointSound;
Effect* dieSound;
Engine *engine;

int random(int min, int max)
{
    return rand() % (max - min + 1) + min;
}

void quit()
{
    delete engine;
    SDL_FreeSurface(background);
    SDL_FreeSurface(pipe);
    SDL_FreeSurface(ground);
    SDL_FreeSurface(bird);
    return;
}

int main(int argc, char* argv[])
{       
    srand(time(NULL));
    engine = new Engine("Flappy Bird", 288, 512);
    engine->initSound();
    engine->targetFrameRate = 60;

    background = SDL_LoadBMP("assets/sprites/background.bmp");
    pipe = SDL_LoadBMP("assets/sprites/pipe.bmp");
    ground = SDL_LoadBMP("assets/sprites/ground.bmp");
    bird = SDL_LoadBMP("assets/sprites/bird.bmp");

    Text* scoreText = new Text("assets/fonts/C64.ttf", 24, { 255, 255, 255, 255 });

    flapSound = new Effect("assets/sounds/wing.wav");
    hitSound = new Effect("assets/sounds/hit.wav");
    pointSound = new Effect("assets/sounds/point.wav");
    dieSound = new Effect("assets/sounds/die.wav");
    flapSound->setVolume(50);
    hitSound->setVolume(50);
    pointSound->setVolume(50);
    dieSound->setVolume(50);
    Sprite* backgroundSprite = new Sprite(engine->renderer, background);
    backgroundSprite->addFrame("background", { 0, 0, 288, 512 });

    Sprite* groundSprite = new Sprite(engine->renderer, ground);
    groundSprite->addFrame("ground", { 0, 0, 336, 112 });
    groundSprite->setPos({ 0, 400 });

    Sprite* pipeSprite = new Sprite(engine->renderer, pipe);
    pipeSprite->addFrame("pipe", { 0, 0, 52, 320 });

    int groundSpeed = 1;
    int groundPosx[2] = { 0, 336 };

    int pipeSpeed = 1;
    int pipeGap = 125;
    int pipeSet1[2] = { 288, random(0, 320 - pipeGap) };
    int pipeSet2[2] = { 288 + 144 + 26, random(0, 320 - pipeGap) };
    
    
    Sprite* birdSprite = new Sprite(engine->renderer, bird);
    AnimationManager* birdAnimations = new AnimationManager(birdSprite);
    std::map<std::string, SDL_Rect> flap = {
            { "up", { 34*2, 0, 34, 24 } },
            { "mid", { 0, 0, 34, 24 } },
            { "down", { 34*1, 0, 34, 24 } },
    };
    std::map<std::string, SDL_Rect> idle = {
            { "mid", { 0, 0, 34, 24 } },
    };
    birdAnimations->addAnimation("flap", flap, 15, false, true, true, true);
    birdAnimations->setAnimation("flap");

    float birdAngle = 0;
    float birdGravity = 0.2;
    float birdVelocity = 0;
    float birdJump = -3.5;
    float birdY = 200;
    bool dead = false;
    int score = 0;
    int jumpDelay = 0;
    bool deathSoundPlayed = false;
    int scoreCooldown = 0;

    scoreText->update("0", engine->renderer);
  
    while (true)
    {

        engine->startFrame();

        engine->update();

        if (engine->input.exit || engine->input.activeKeys[SDLK_ESCAPE])
        {
            quit();
        } 

        birdVelocity += birdGravity;

        if (engine->input.activeKeys[SDLK_SPACE] && !dead && jumpDelay == 0)
        {
            birdVelocity = birdJump;
            flapSound->play();
            jumpDelay = 10;
        }

        if (jumpDelay > 0)
        {
            jumpDelay--;
        }

        birdY += birdVelocity;
        birdAngle = birdVelocity * 3;

        engine->clear({ 0, 0, 0, 255 });
        backgroundSprite->draw(engine->renderer, "background");

        pipeSprite->setPos({ pipeSet1[0], pipeSet1[1] - 320 });
        pipeSprite->draw(engine->renderer, "pipe", true, SDL_FLIP_VERTICAL);
        pipeSprite->setPos({ pipeSet1[0], pipeSet1[1] + pipeGap });
        pipeSprite->draw(engine->renderer, "pipe");

        pipeSprite->setPos({ pipeSet2[0], pipeSet2[1] - 320 });
        pipeSprite->draw(engine->renderer, "pipe", true, SDL_FLIP_VERTICAL);
        pipeSprite->setPos({ pipeSet2[0], pipeSet2[1] + pipeGap });
        pipeSprite->draw(engine->renderer, "pipe");

        pipeSet1[0] -= pipeSpeed;
        pipeSet2[0] -= pipeSpeed;
      
        if (birdY >= 400 || birdY <= 0)
        {
            dead = true;
        }

        if (birdSprite->dstrect.x + birdSprite->dstrect.w >= pipeSet1[0] && birdSprite->dstrect.x <= pipeSet1[0] + 52)
        {
            if (birdSprite->dstrect.y <= pipeSet1[1] || birdSprite->dstrect.y + birdSprite->dstrect.h >= pipeSet1[1] + pipeGap)
            {
                dead = true;
            }
        }

        if (birdSprite->dstrect.x + birdSprite->dstrect.w >= pipeSet2[0] && birdSprite->dstrect.x <= pipeSet2[0] + 52)
        {
            if (birdSprite->dstrect.y <= pipeSet2[1] || birdSprite->dstrect.y + birdSprite->dstrect.h >= pipeSet2[1] + pipeGap)
            {
                dead = true;
            }
        }

        if (birdSprite->dstrect.x >= pipeSet1[0] + 26 - 24 && birdSprite->dstrect.x <= pipeSet1[0] + 26 - 24 + pipeSpeed && !dead && scoreCooldown == 0)
        {
            score++;
            scoreCooldown = 5;
            pointSound->play();
            scoreText->update(std::to_string(score), engine->renderer);
        }

        if (birdSprite->dstrect.x >= pipeSet2[0] + 26 - 24 && birdSprite->dstrect.x <= pipeSet2[0] + 26 - 24 + pipeSpeed && !dead && scoreCooldown == 0)
        {
            score++;
            scoreCooldown = 5;
            pointSound->play();
            scoreText->update(std::to_string(score), engine->renderer);
        }

        if (scoreCooldown > 0)
        {
            scoreCooldown--;
        }

        if (dead && !deathSoundPlayed)
        {
            pipeSpeed = 0;
            groundSpeed = 0;
            hitSound->play();
            dieSound->play();
            deathSoundPlayed = true;
        }

        if (pipeSet1[0] <= -52)
        {
            pipeSet1[0] = 288;
            pipeSet1[1] = random(120, 320 - pipeGap);
        }

        if (pipeSet2[0] <= -52)
        {
            pipeSet2[0] = 288;
            pipeSet2[1] = random(120, 320 - pipeGap);
        }

        birdSprite->angle = birdAngle;
        birdSprite->setPos({ 50, (int)birdY });
        birdAnimations->update();
        birdAnimations->draw(engine->renderer);

        for (int i = 0; i < 2; i++)
        {
            groundSprite->setPos({ groundPosx[i], 400 });
            groundSprite->draw(engine->renderer, "ground");
            groundPosx[i] -= groundSpeed;
            if (groundPosx[i] <= -336)
            {
                groundPosx[i] = 336;
            }
        }

        scoreText->draw(engine->renderer, 144, 50);
        engine->render();
    }
    quit();
    return 0;
}